{******************************************************************************************}
{ "Hundir la Flota", recreacion del clasico juego de tablero de guerra maritima.           }
{ Copyright (C) 2002  Gorka Surez Garca                                                  }
{                                                                                          }
{ Graflib.pas is part of "Hundir la Flota".                                                }
{                                                                                          }
{ "Hundir la Flota" is free software; you can redistribute it and/or                       }
{ modify it under the terms of the GNU General Public License                              }
{ as published by the Free Software Foundation; either version 2                           }
{ of the License, or (at your option) any later version.                                   }
{                                                                                          }
{ "Hundir la Flota" is distributed in the hope that it will be useful,                     }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                           }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            }
{ GNU General Public License for more details.                                             }
{                                                                                          }
{ You should have received a copy of the GNU General Public License                        }
{ along with "Hundir la Flota"; if not, write to the Free Software                         }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              }
{******************************************************************************************}

{**************************************************************************}
{ Graflib.pas                                                              }
{ Autor: Gorka Suarez Garcia                                               }
{ Fecha: 04-02-02                                                          }
{**************************************************************************}
Unit Graflib;
Interface {Parte publica}
   Uses Crt;

   Const
    {Colores para el color principal y el color de fondo}
      Negro        = Black;
      Azul         = Blue;
      Verde        = Green;
      {Cyan         = Cyan;}
      Rojo         = Red;
      {Magenta      = Magenta;}
      Marron       = Brown;
      GrisClaro    = LightGray;
    {Colores para el color principal}
      GrisOscuro   = DarkGray;
      AzulClaro    = LightBlue;
      VerdeClaro   = LightGreen;
      CyanClaro    = LightCyan;
      RojoClaro    = LightRed;
      MagentaClaro = LightMagenta;
      Amarillo     = Yellow;
      Blanco       = White;
      Parpadeo     = Blink;
    {Maximos de la pantalla}
      MaxX = 80;
      MaxY = 49;
    {Varias}
      UltF = 50; {La ultima fila de la pantalla, para escribir mensajes, sin
                  tener que meterlos en el buffer de pantalla}
      CBLANCO = ''; {La uso para poder rellenar zonas de la pantalla con
                      colores que el color de fondo no admite}

   Type
      TPunto = Record
         Car : Char;
         Col : Byte; {Distribucion de los colores TTTTFFFF:
                      - TTTT = Color del Texto
                      - FFFF = Color del Fondo
                      Uso esto para ahorrar memoria, ya que si usase una
                      variable para cada color, se estarian desaprobechando
                      en cada una 4bits, con lo que se desaprobecharian
                      1byte (y eso no es demasiado bueno)}
      End; {Emulacion de un Pixel con texto}

{**************************************************************************}

   Procedure IniciarModo8050; {Para usar esta libreria se ha de llamar a
                               este procedimiento que inicializa la libreria
                               y el modo de pantalla 80x50x16}
   Procedure QuitarModo8050; {Quita el modo de pantalla de 80x50x16}


   Procedure CambiarColor (c : Byte); {Cambia el color actual de la
                           pantalla}
   Procedure Repintado (x, y, x2, y2 : Byte); {Repinta una region de la
                        pantalla, con el contenido del buffer}
   Procedure RepintadoRapido (x, y, x2, y2 : Byte); {Lo mismo que la
                              anterior pero sin tener en los colores
                              indicados en el buffer}
   Procedure BorrarBuffer; {Borra el buffer}
   Procedure BorrarPantalla; {Borra el buffer y la pantalla}

   Procedure ActualizarPantalla; {Pinta todo el contenido del buffer en
                                  Pantalla, con Repintado}
   Procedure ActualizarPantallaRapido; {Lo mismo que la anterior, pero esta
                                        usa RepintadoRapido}


   {Estos procedimientos pintan sobre el buffer}
   Procedure PintarPunto (p : TPunto; x, y : Byte); {Pinta un punto}
   Procedure PintarLineaH (p : TPunto; x, y, x2 : Byte); {Pinta una linea
                                                          Horizontal}
   Procedure PintarLineaV (p : TPunto; x, y, y2 : Byte); {Pinta una linea
                                                          Vertical}
   Procedure PintarRectangulo (p : TPunto; x, y, x2, y2 : Byte); {Pinta un
                                                                 Rectangulo}
   Procedure PintarTexto (t : String; col, x, y : Byte); {Pinta un texto}
   Procedure PintarPunto2 (car : Char; col, x, y : Byte);
   Procedure PintarLineaH2 (car : Char; col, x, y, x2 : Byte);
   Procedure PintarLineaV2 (car : Char; col, x, y, y2 : Byte);
   Procedure PintarRectangulo2 (car : Char; col, x, y, x2, y2 : Byte);


   Function  Color (c, b : Byte) : Byte; {Devuelve el color que estara
                                          compuesto por el color de texto y
                                          el de fondo}
   Procedure DarPunto (x, y : Byte; var p : TPunto); {Devuelve el punto del
                       buffer que le indiquemos}

{**************************************************************************}

Implementation {Parte privada}
   Type
      TPantalla = array [1..MaxY, 1..MaxX] of TPunto;

   Var
      Pantalla : TPantalla; {buffer de la pantalla}
      {Porque usar un buffer y no hacerlo directamente sobre la pantalla?
          Pues muy sencillo, con un buffer auxiliar, puedo componer una
          imagen y cuando esta este preparada, la pinto sobre la pantalla.
          Si no se hiciera esto, podria ser traumatico ver como se pinta
          primero un cuadrado y luego una linea, etc.
      }

{**************************************************************************}

   Procedure IniciarModo8050;
   Begin
      TextMode(CO80+Font8x8);
      BorrarPantalla;
   End;


   Procedure QuitarModo8050;
   Begin
      TextMode(CO80);
   End;

{**************************************************************************}

   Procedure CambiarColor (c : Byte);
   Begin
      TextColor(c Div 16);
      TextBackground(c Mod 16);
   End;

{**************************************************************************}

   Procedure Repintado (x, y, x2, y2 : Byte);
   Var
      f, c : Byte;
   Begin
      {Gotoxy(1, 1);}
      For f := y To y2 Do
       Begin
         GotoXY(1, f);
         For c := x To x2 Do
          Begin
            CambiarColor(Pantalla[f, c].Col);
            Write(Pantalla[f, c].Car);
          End;
       End;
   End;


   Procedure RepintadoRapido (x, y, x2, y2 : Byte);
   Var
      f, c : Byte;
   Begin
      {Gotoxy(1, 1);}
      For f := y To y2 Do
       Begin
         GotoXY(1, f);
         For c := x To x2 Do
            Write(Pantalla[f, c].Car);
       End;
   End;

{**************************************************************************}

   Procedure BorrarBuffer;
   Var
      f, c : Byte;
   Begin
      For f := 1 To MaxY Do
         For c := 1 To MaxX Do
          Begin
            Pantalla[f, c].Col := 0;
            Pantalla[f, c].Car := ' ';
          End;
   End;


   Procedure BorrarPantalla;
   Begin
      BorrarBuffer;
      CambiarColor(0);
      ClrScr;
   End;

{**************************************************************************}

   Procedure ActualizarPantalla;
   Begin
      Repintado(1, 1, MaxX, MaxY);
   End;


   Procedure ActualizarPantallaRapido;
   Begin
      RepintadoRapido(1, 1, MaxX, MaxY);
   End;

{**************************************************************************}

   Procedure PintarPunto (p : TPunto; x, y : Byte);
   Begin
      Pantalla[y, x] := p;
   End;


   Procedure PintarLineaH (p : TPunto; x, y, x2 : Byte);
   Var
      c : Byte;
   Begin
      For c := x To x2 Do
         Pantalla[y, c] := p;
   End;


   Procedure PintarLineaV (p : TPunto; x, y, y2 : Byte);
   Var
      f : Byte;
   Begin
      For f := y To y2 Do
         Pantalla[f, x] := p;
   End;


   Procedure PintarRectangulo (p : TPunto; x, y, x2, y2 : Byte);
   Var
      f, c : Byte;
   Begin
      For f := y To y2 Do
         For c := x To x2 Do
            Pantalla[f, c] := p;
   End;


   Procedure PintarTexto (t : String; col, x, y : Byte);
   Var
      c : Byte;
   Begin
         For c := 1 To Length(t) Do
          Begin
            Pantalla[y, (c + x - 1)].Col := col;
            Pantalla[y, (c + x - 1)].Car := t[c];
          End;
   End;


   Procedure PintarPunto2 (car : Char; col, x, y : Byte);
   Begin
      Pantalla[y, x].Car := car;
      Pantalla[y, x].Col := col;
   End;


   Procedure PintarLineaH2 (car : Char; col, x, y, x2 : Byte);
   Var
      c : Byte;
   Begin
      For c := x To x2 Do
       Begin
         Pantalla[y, c].Car := car;
         Pantalla[y, c].Col := col;
       End;
   End;


   Procedure PintarLineaV2 (car : Char; col, x, y, y2 : Byte);
   Var
      f : Byte;
   Begin
      For f := y To y2 Do
       Begin
         Pantalla[f, x].Car := car;
         Pantalla[f, x].Col := col;
       End;
   End;


   Procedure PintarRectangulo2 (car : Char; col, x, y, x2, y2 : Byte);
   Var
      f, c : Byte;
   Begin
      For f := y To y2 Do
         For c := x To x2 Do
          Begin
            Pantalla[f, c].Car := car;
            Pantalla[f, c].Col := col;
          End;
   End;

{**************************************************************************}

   Function Color (c, b : Byte) : Byte;
   Begin
      Color := (c * 16) + b;
   End;


   Procedure DarPunto (x, y : Byte; var p : TPunto);
   Begin
      p := Pantalla[y, x];
   End;

{**************************************************************************}

End.

{**************************************************************************}
{                         Fin de la unidad Graflib                         }
{**************************************************************************}