{******************************************************************************************}
{ "Hundir la Flota", recreacion del clasico juego de tablero de guerra maritima.           }
{ Copyright (C) 2002  Gorka Surez Garca                                                  }
{                                                                                          }
{ Juego.pas is part of "Hundir la Flota".                                                  }
{                                                                                          }
{ "Hundir la Flota" is free software; you can redistribute it and/or                       }
{ modify it under the terms of the GNU General Public License                              }
{ as published by the Free Software Foundation; either version 2                           }
{ of the License, or (at your option) any later version.                                   }
{                                                                                          }
{ "Hundir la Flota" is distributed in the hope that it will be useful,                     }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                           }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            }
{ GNU General Public License for more details.                                             }
{                                                                                          }
{ You should have received a copy of the GNU General Public License                        }
{ along with "Hundir la Flota"; if not, write to the Free Software                         }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              }
{******************************************************************************************}

{**************************************************************************}
{ Juego.pas                                                                }
{ Autor: Gorka Suarez Garcia                                               }
{ Fecha: 06-02-02                                                          }
{**************************************************************************}
Unit Juego;
Interface {Parte publica}
   Uses Crt, Graficos, Graflib, Puntos;

   Procedure UnJugador;
   Procedure DosJugadores;

{**************************************************************************}

Implementation {Parte privada}
{----- Tipos -----}
   Type
      {Tipos para manejar los jugadores}
      TBarcos = array [1..10] of Record
         f, c, p, t : Byte;
      End;
      TTablero = array [1..10, 1..10] of Byte;
      TJugador = Record
         Nombre : String[16];
         Disparos : Byte;
         NumBarcos : Byte;
         Tablero : TTablero;
      End;

      TMaquina = Record
         NumBarcos : Byte;
         Estado : Byte;
         Mdx, Mdy : Byte;
         Tablero : TTablero;
      End;


{**************************************************************************}
{----------------------- Funciones y procedimientos -----------------------}
{**************************************************************************}

   Function PosValida (b : TBarcos; n : Byte) : Boolean;
   Var
      Valido : Boolean;
      i, j : Byte;
   Begin
      Valido := True; i := 1;

      while (i < n) And Valido do
       begin
         j := 0;
         If (b[i].p = 0) And (b[n].p = 0) Then {HH}
            while (j < b[n].t) And Valido do
             begin
               Valido := Not((b[n].f In [(b[i].f-1)..(b[i].f+1)]) And
                             ((b[n].c+j) In [(b[i].c-1)..(b[i].c+b[i].t)]));
               j := j + 1;
             end
         Else If (b[i].p = 0) And (b[n].p = 1) Then {HV}
            while (j < b[n].t) And Valido do
             begin
               Valido := Not(((b[n].f+j) In [(b[i].f-1)..(b[i].f+1)]) And
                             (b[n].c In [(b[i].c-1)..(b[i].c+b[i].t)]));
               j := j + 1;
             end
         Else If (b[i].p = 1) And (b[n].p = 0) Then {VH}
            while (j < b[n].t) And Valido do
             begin
               Valido := Not((b[n].f In [(b[i].f-1)..(b[i].f+b[i].t)]) And
                             ((b[n].c+j) In [(b[i].c-1)..(b[i].c+1)]));
               j := j + 1;
             end
         Else If (b[i].p = 1) And (b[n].p = 1) Then {VV}
            while (j < b[n].t) And Valido do
             begin
               Valido := Not(((b[n].f+j) In [(b[i].f-1)..(b[i].f+b[i].t)]) And
                             (b[n].c In [(b[i].c-1)..(b[i].c+1)]));
               j := j + 1;
             end;
         i := i + 1;
       end;

      PosValida := Valido;
   End;

{**************************************************************************}

   Procedure PintarTableroBarcos (T : TTablero);
   Var
      f, c, col : Byte;
   Begin
      For f := 1 To 10 Do
         For c := 1 To 10 Do
          Begin
            Case T[f, c] Of
               0: col := Color(Negro, Negro); {1-4 H, 5-8 V}
               1..8: col := Color(GrisOscuro, Negro);
               9: col := Color(AzulClaro, Negro);
               11..18: col := Color(RojoClaro, Negro); {11-14 H, 15-18 V}
            End;
            PintarRectangulo2(CBLANCO, col, (8+3*(c-1)), (7+3*(f-1)), (10+3*(c-1)), (9+3*(f-1)));
          End;
   End;

{**************************************************************************}

   Procedure PintarTableroDisparos (T : TTablero);
   Var
      f, c, col : Byte;
   Begin
      For f := 1 To 10 Do
         For c := 1 To 10 Do
          Begin
            Case T[f, c] Of
               0..8: col := Color(Negro, Negro);
               9: col := Color(AzulClaro, Negro);
               11..18: col := Color(RojoClaro, Negro); {11-14 H, 15-18 V}
            End;
            PintarRectangulo2(CBLANCO, col, (44+3*(c-1)), (7+3*(f-1)), (46+3*(c-1)), (9+3*(f-1)));
          End;
   End;

{**************************************************************************}

   Procedure MeterCoordenadas (var f, c : Byte);
   Var
      Cad : String[3];
      Valido : Boolean;
      Cod : Integer;
      i : Byte;
   Begin
      CambiarColor(Color(Amarillo, Negro));
      Valido := True;

      Repeat
         GotoXY(1, 50); ClrEol;
         Repeat
            Cad := '';
            GotoXY(1, 49); ClrEol;
            Write('Meta las coordenadas (Ejemplo: A10): ');         
            readln(Cad);
         Until Cad <> '';

         Val(Copy(Cad, 2, (Length(Cad) - 1)), c, Cod);
         If Cod = 0 Then
            Valido := ((Upcase(Cad[1]) In ['A'..'J']) And (c In [1..10]))
         Else
            Valido := False;

         If Valido Then
            f := (Ord(Upcase(Cad[1])) - Ord('A') + 1)
         Else
          Begin
            GotoXY(1, 50);
            Write(Cad, ' es una coordenada erronea...');
            ReadKey;
          End;
      Until Valido;
   End;

{**************************************************************************}

   Procedure ColocacionManual (var T : TTablero; N : String);
   Var
      Respuesta : Char;
      Col, N1, N2, N3, N4, nb, i : Byte;
      Barcos : TBarcos;
      Salir, Valido : Boolean;
   Begin
      ClrScr;
      Col := Color(VerdeClaro, Negro);
      PintarTexto('                ', Col, 9, 42);
      PintarTexto(N, Col, 9, 42);
      PintarTexto('  0', Col, 22, 44);
      ActualizarPantalla;

      Col := Color(Amarillo, Negro);
      CambiarColor(Col);

      N1 := 4; N2 := 3;
      N3 := 2; N4 := 1;
      nb := 1;

      Repeat
         Repeat
            GotoXY(1, 50);
            Write('Indique el barco a poner: 1-Lanchas/2-Fragata/3-Cruceros/4-Porta-Aviones? ');
            ClrEol;

            Valido := False;
            Respuesta := ReadKey;
            If (N1 > 0) And (Respuesta = '1') Then Valido := True;
            If (N2 > 0) And (Respuesta = '2') Then Valido := True;
            If (N3 > 0) And (Respuesta = '3') Then Valido := True;
            If (N4 > 0) And (Respuesta = '4') Then Valido := True;

            If Not Valido Then
             Begin
               GotoXY(1, 50);
               Write('No quedan de ese tipo de barcos, pruebe con otro tipo...');
               ClrEol;
               ReadKey;
             End;
         Until Valido;
         Barcos[nb].t := (Ord(Respuesta) - Ord('1') + 1);

         Repeat
            GotoXY(1, 50);
            Write('Lo va a poner en Horizontal o Vertical (H/V)? ');
            ClrEol;
            Repeat
               Respuesta := UpCase(ReadKey);
            Until Respuesta In ['H', 'V'];
            If Respuesta = 'H' Then
               Barcos[nb].p := 0
            Else
               Barcos[nb].p := 1;

            MeterCoordenadas(Barcos[nb].f, Barcos[nb].c);
            If (Barcos[nb].p = 0) And (Barcos[nb].c > (11 - Barcos[nb].t)) Then
               Barcos[nb].c := (11 - Barcos[nb].t);

            If (Barcos[nb].p = 1) And (Barcos[nb].f > (11 - Barcos[nb].t)) Then
               Barcos[nb].f := (11 - Barcos[nb].t);

            Valido := PosValida(Barcos, nb);
            If Not Valido Then
             Begin
               GotoXY(1, 50);
               Write('No se puede colocar ningun barco pegado a otro!!!');
               ClrEol;
               Readkey;
             End;
         Until Valido;


         GotoXY(1, 50);
         Write('Esta seguro de poner ese barco ahi (S/N)?');
         ClrEol;
         Repeat
            Respuesta := UpCase(ReadKey);
         Until Respuesta In ['S', 'N'];

         If Respuesta = 'S' Then
          Begin
            Case Barcos[nb].t Of
               1:
                Begin
                  T[Barcos[nb].f, Barcos[nb].c] := 1;
                  N1 := N1 - 1;
                End;
               2:
                Begin
                  If Barcos[nb].p = 0 Then
                     For i := 0 To 1 Do
                        T[Barcos[nb].f, (Barcos[nb].c + i)] := 2
                  Else
                     For i := 0 To 1 Do
                        T[(Barcos[nb].f + i), Barcos[nb].c] := 6;
                  N2 := N2 - 1;
                End;
               3:
                Begin
                  If Barcos[nb].p = 0 Then
                     For i := 0 To 2 Do
                        T[Barcos[nb].f, (Barcos[nb].c + i)] := 3
                  Else
                     For i := 0 To 2 Do
                        T[(Barcos[nb].f + i), Barcos[nb].c] := 7;
                  N3 := N3 - 1;
                End;
               4:
                Begin
                  If Barcos[nb].p = 0 Then
                     For i := 0 To 3 Do
                        T[Barcos[nb].f, (Barcos[nb].c + i)] := 4
                  Else
                     For i := 0 To 3 Do
                        T[(Barcos[nb].f + i), Barcos[nb].c] := 8;
                  N4 := N4 - 1;
                End;
            End;

            nb := nb + 1;
            PintarTableroBarcos(T);
            ActualizarPantalla;
            CambiarColor(Col);
          End;

         Salir := (N1 <= 0) And (N2 <= 0) And (N3 <= 0) And (N4 <= 0);
      Until Salir;
   End;

{**************************************************************************}

   Procedure ColocacionAutomatica (var T : TTablero);
   Var
      i, j, nb : Byte;
      Barcos : TBarcos;
   Begin
      nb := 1;
      {Porta-aviones}
      Barcos[nb].t := 4;
      Barcos[nb].p := Random(2);
      If Barcos[nb].p = 0 Then {Horizontal}
       Begin
         Barcos[nb].f := Random(10) + 1;
         Barcos[nb].c := Random(7) + 1;
         For i := 0 To 3 Do
            T[Barcos[nb].f, (Barcos[nb].c + i)] := 4;
       End
      Else {Vertical}
       Begin
         Barcos[nb].c := Random(10) + 1;
         Barcos[nb].f := Random(7) + 1;
         For i := 0 To 3 Do
            T[(Barcos[nb].f + i), Barcos[nb].c] := 8;
       End;

      {Cruceros}
      For i := 1 To 2 Do
       Begin
         nb := nb + 1;
         Barcos[nb].t := 3;
         Repeat
            Barcos[nb].p := Random(2);
            If Barcos[nb].p = 0 Then {Horizontal}
             Begin
               Barcos[nb].f := Random(10) + 1;
               Barcos[nb].c := Random(8) + 1;
             End
            Else {Vertical}
             Begin
               Barcos[nb].c := Random(10) + 1;
               Barcos[nb].f := Random(8) + 1;
             End;
         Until PosValida(Barcos, nb);

         If Barcos[nb].p = 0 Then {Horizontal}
            For j := 0 To 2 Do
               T[Barcos[nb].f, (Barcos[nb].c + j)] := 3
         Else {Vertical}
            For j := 0 To 2 Do
               T[(Barcos[nb].f + j), Barcos[nb].c] := 7;
       End;

      {Fragatas}
      For i := 1 To 3 Do
       Begin
         nb := nb + 1;
         Barcos[nb].t := 2;
         Repeat
            Barcos[nb].p := Random(2);
            If Barcos[nb].p = 0 Then {Horizontal}
             Begin
               Barcos[nb].f := Random(10) + 1;
               Barcos[nb].c := Random(9) + 1;
             End
            Else {Vertical}
             Begin
               Barcos[nb].c := Random(10) + 1;
               Barcos[nb].f := Random(9) + 1;
             End;
         Until PosValida(Barcos, nb);

         If Barcos[nb].p = 0 Then {Horizontal}
            For j := 0 To 1 Do
               T[Barcos[nb].f, (Barcos[nb].c + j)] := 2
         Else {Vertical}
            For j := 0 To 1 Do
               T[(Barcos[nb].f + j), Barcos[nb].c] := 6;
       End;

      {Lanchas}
      For i := 1 To 4 Do
       Begin
         nb := nb + 1;
         Barcos[nb].t := 1;
         Barcos[nb].p := 0;
         Repeat
            Barcos[nb].f := Random(10) + 1;
            Barcos[nb].c := Random(10) + 1;
         Until PosValida(Barcos, nb);

         T[Barcos[nb].f, Barcos[nb].c] := 1;
       End;

   End;

{**************************************************************************}

   Procedure InitJugador (var Jugador : TJugador; N : Byte);
   Var
      Respuesta : Char;
      f, c : Byte;
   Begin
      CambiarColor(Color(VerdeClaro, Negro));
      ClrScr;
      GotoXY(1, 50);
      Case N Of
         0: Write('Meta su nombre jugador (maximo 16 caracteres): ');
         1: Write('Meta su nombre jugador 1 (maximo 16 caracteres): ');
         2: Write('Meta su nombre jugador 2 (maximo 16 caracteres): ');
      End;
      Readln(Jugador.Nombre);
      Jugador.Disparos := 0;
      Jugador.NumBarcos := 10;

      ClrScr;
      GotoXY(1, 50);
      Write('Desea colocar los barcos Automaticamente o Manualmente (A/M)? ');
      Repeat
         GotoXY(64, 50);
         Respuesta := Upcase(ReadKey);
      Until Respuesta In ['A', 'M'];

      For f := 1 To 10 Do
         For c := 1 To 10 Do
            Jugador.Tablero[f, c] := 0;

      If Respuesta = 'A' Then
         ColocacionAutomatica(Jugador.Tablero)
      Else
         ColocacionManual(Jugador.Tablero, Jugador.Nombre);

   End;

{**************************************************************************}

   Procedure InitMaquina (var Maquina : TMaquina);
   Var
      f, c : Byte;
   Begin
      Maquina.NumBarcos := 10;
      Maquina.Estado := 0;
      Maquina.Mdx := 0;
      Maquina.Mdy := 0;

      For f := 1 To 10 Do
         For c := 1 To 10 Do
            Maquina.Tablero[f, c] := 0;

      ColocacionAutomatica(Maquina.Tablero);

   End;

{**************************************************************************}

   Function Hundido(f, c : Byte; T : TTablero) : Boolean;
   Var
      Valido : Boolean;
      i, tam : Byte;
   Begin
      Valido := True;
      Case T[f, c] Of
         11, 15: Valido := True;
         12..14: {H}
          Begin
            tam := T[f, c] - 11;

            i := c - 1;
            while (i >= (c - tam)) And (T[f, i] <> 9) Do {Recorre lo que hay a la izquierda del punto}
             begin
               If (i In [1..10]) And Valido Then
                  Valido := Not(T[f, i] In [2..4]);
               i := i - 1;
             end;

            i := c + 1;
            while (i <= (c + tam)) And (T[f, i] <> 9) Do {Recorre lo que hay a la derecha del punto}
             begin
               If (i In [1..10]) And Valido Then
                  Valido := Not(T[f, i] In [2..4]);
               i := i + 1;
             end;
          End;
         16..18: {V}
          Begin
            tam := T[f, c] - 15;

            i := f - 1;
            while (i >= (f - tam)) And (T[i, c] <> 9) Do {Recorre lo que hay arriba del punto}
             begin
               If (i In [1..10]) And Valido Then
                  Valido := Not(T[i, c] In [6..8]);
               i := i - 1;
             end;

            i := f + 1;
            while (i <= (f + tam)) And (T[i, c] <> 9) Do {Recorre lo que hay abajo del punto}
             begin
               If (i In [1..10]) And Valido Then
                  Valido := Not(T[i, c] In [6..8]);
               i := i + 1;
             end;
          End;
      End;
      Hundido := Valido;
   End;

{**************************************************************************}

   Procedure Turno (var Jugador: TJugador; var Tablero : TTablero; var NumBarcos : Byte);
   Var
      Col, i, f, c : Byte;
      Disp : String;
   Begin
      CambiarColor(0);
      ClrScr;

      Col := Color(VerdeClaro, Negro);
      PintarTexto('                ', Col, 9, 42);
      PintarTexto(Jugador.Nombre, Col, 9, 42);

      For i := 1 To 3 Do
       If NumBarcos > 0 Then
       Begin
         PintarTableroBarcos(Jugador.Tablero);
         PintarTableroDisparos(Tablero);

         Str(Jugador.Disparos, Disp);
         Case Length(Disp) Of
            1: Disp := '  ' + Disp;
            2: Disp := ' ' + Disp;
         End;
         PintarTexto(Disp, Col, 22, 44);

         ActualizarPantalla;

         Repeat
            MeterCoordenadas(f, c);
            If Not(Tablero[f, c] In [0..8]) Then
             Begin
               GotoXY(1, 50);
               Write('Ya has disparado sobre: ', Chr(f-1+ord('A')), c, '...');
               ClrEol;
               ReadKey;
             End;
         Until Tablero[f, c] In [0..8];

         Jugador.Disparos := Jugador.Disparos + 1;

         Case Tablero[f, c] Of
            0:
             Begin
               Tablero[f, c] := 9;
               CambiarColor(Color(AzulClaro, Negro));
               GotoXY(1, 50);
               Write('Agua...');
               ClrEol;
               ReadKey;
             End;

            1..8:
             Begin                  
               Tablero[f, c] := Tablero[f, c] + 10;
               CambiarColor(Color(RojoClaro, Negro));
               GotoXY(1, 50);

               If Hundido(f, c, Tablero) Then
                Begin
                  NumBarcos := NumBarcos - 1;
                  Case Tablero[f, c] Of
                     11, 15: Write('Lancha hundida!!! Quedan ', NumBarcos, ' Barcos...');
                     12, 16: Write('Crucero hundido!!! Quedan ', NumBarcos, ' Barcos...');
                     13, 17: Write('Fragata hundida!!! Quedan ', NumBarcos, ' Barcos...');
                     14, 18: Write('Porta-aviones hundido!!! Quedan ', NumBarcos, ' Barcos...');
                  End;
                End
               Else
                  Write('Tocado!!!');

               ClrEol;
               ReadKey;
             End;
         End;
       End;
   End;

{**************************************************************************}

   Procedure MaquinaHundeBarco (var Jugador : TJugador; var Maquina : TMaquina);
   Begin
      Jugador.NumBarcos := Jugador.NumBarcos - 1;
      Maquina.Estado := 0;
   End;

{**************************************************************************}

   Function TiroMaquinaValido (c, f : Byte; Jugador : TJugador) : Boolean;
   Var
      Valido : Boolean;
   Begin
      Valido := True;

      If ((f-1) > 0) And ((c-1) > 0) And Valido Then
         Valido := Not(Jugador.Tablero[(f-1), (c-1)] In [11..18]);

      If ((f-1) > 0) And Valido Then
         Valido := Not(Jugador.Tablero[(f-1), c] In [11..18]);

      If ((f-1) > 0) And ((c+1) < 11) And Valido Then
         Valido := Not(Jugador.Tablero[(f-1), (c+1)] In [11..18]);

      If ((c-1) > 0) And Valido Then
         Valido := Not(Jugador.Tablero[f, (c-1)] In [11..18]);

      If Valido Then
         Valido := (Jugador.Tablero[f, c] In [0..8]);

      If ((c+1) < 11) And Valido Then
         Valido := Not(Jugador.Tablero[f, (c+1)] In [11..18]);

      If ((f+1) < 11) And ((c-1) > 0) And Valido Then
         Valido := Not(Jugador.Tablero[(f+1), (c-1)] In [11..18]);

      If ((f+1) < 11) And Valido Then
         Valido := Not(Jugador.Tablero[(f+1), c] In [11..18]);

      If ((f+1) < 11) And ((c+1) < 11) And Valido Then
         Valido := Not(Jugador.Tablero[(f+1), (c+1)] In [11..18]);

      TiroMaquinaValido := Valido;
   End;

{**************************************************************************}

   Function Estado00 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
   Begin {Randomize;}
      Repeat
         f := Random(10) + 1;
         c := Random(10) + 1;
      Until TiroMaquinaValido(c, f, Jugador);

      Case Jugador.Tablero[f, c] Of
         0:
          Begin
            Jugador.Tablero[f, c] := 9;
            Maquina.Estado := 0;
          End;
         1..8:
          Begin
           Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
           If Hundido(f, c, Jugador.Tablero) Then {Hundido}
              MaquinaHundeBarco(Jugador, Maquina)
           Else {Tocado}
            Begin
              Maquina.Estado := 1;
              Maquina.Mdx := c;
              Maquina.Mdy := f;
            End;
         End;
      End;

      Estado00 := True;
   End;

{**************************************************************************}

   Function Estado01 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy - 1;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 2
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 2;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 11;
             End;
         End;
       End;

      Estado01 := Salir;
   End;

{**************************************************************************}

   Function Estado11 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy - 2;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 3
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 3;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 21;
             End;
         End;
       End;

      Estado11 := Salir;
   End;

{**************************************************************************}

   Function Estado21 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy - 3;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 3
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 3;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina);
             End;
         End;
       End;

      Estado21 := Salir;
   End;

{**************************************************************************}

   Function Estado02 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx + 1;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 3
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 3;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 12;
             End;
         End;
       End;

      Estado02 := Salir;
   End;

{**************************************************************************}

   Function Estado12 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx + 2;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 4
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 4;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 22;
             End;
         End;
       End;

      Estado12 := Salir;
   End;

{**************************************************************************}

   Function Estado22 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx + 3;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 4
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 4;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina);
             End;
         End;
       End;

      Estado22 := Salir;
   End;

{**************************************************************************}

   Function Estado03 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy + 1;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 4
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 4;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 13;
             End;
         End;
       End;

      Estado03 := Salir;
   End;

{**************************************************************************}

   Function Estado13 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy + 2;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 1
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 1;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 23;
             End;
         End;
       End;

      Estado13 := Salir;
   End;

{**************************************************************************}

   Function Estado23 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx;
      f := Maquina.Mdy + 3;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 1
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 1;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina);
             End;
         End;
       End;

      Estado23 := Salir;
   End;

{**************************************************************************}

   Function Estado04 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx - 1;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 0
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 0;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 14;
             End;
         End;
       End;

      Estado04 := Salir;
   End;

{**************************************************************************}

   Function Estado14 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx - 2;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 2
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 2;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina)
               Else {Tocado}
                  Maquina.Estado := 24;
             End;
         End;
       End;

      Estado14 := Salir;
   End;

{**************************************************************************}

   Function Estado24 (var Maquina : TMaquina; var Jugador : TJugador) : Boolean;
   Var
      f, c : Byte;
      Salir : Boolean;
   Begin
      c := Maquina.Mdx - 3;
      f := Maquina.Mdy;

      Salir := (f In [1..10]) And (c In [1..10]) And (Jugador.Tablero[f, c] In [0..8]);
      If Not Salir Then
         Maquina.Estado := 2
      Else
       Begin
         Case Jugador.Tablero[f, c] Of
            0:
             Begin
               Jugador.Tablero[f, c] := 9;
               Maquina.Estado := 2;
             End;
            1..8:
             Begin
               Jugador.Tablero[f, c] := Jugador.Tablero[f, c] + 10;
               If Hundido(f, c, Jugador.Tablero) Then {Hundido}
                  MaquinaHundeBarco(Jugador, Maquina);
             End;
         End;
       End;

      Estado24 := Salir;
   End;

{**************************************************************************}

   Procedure TurnoMaquina (var Maquina : TMaquina; var Jugador : TJugador);
   Var
      i : Byte;
      Salir : Boolean;
   Begin
      For i := 1 To 3 Do
         If Jugador.NumBarcos > 0 Then
            Repeat
               Salir := True;

               Case Maquina.Estado Of
                  0:  Salir := Estado00(Maquina, Jugador);
                  1:  Salir := Estado01(Maquina, Jugador);
                  11: Salir := Estado11(Maquina, Jugador);
                  21: Salir := Estado21(Maquina, Jugador);
                  2:  Salir := Estado02(Maquina, Jugador);
                  12: Salir := Estado12(Maquina, Jugador);
                  22: Salir := Estado22(Maquina, Jugador);
                  3:  Salir := Estado03(Maquina, Jugador);
                  13: Salir := Estado13(Maquina, Jugador);
                  23: Salir := Estado23(Maquina, Jugador);
                  4:  Salir := Estado04(Maquina, Jugador);
                  14: Salir := Estado14(Maquina, Jugador);
                  24: Salir := Estado24(Maquina, Jugador);
               End;
            Until Salir;
   End;

{**************************************************************************}

   Procedure UnJugador;
   Var
      Jugador : TJugador;
      Maquina : TMaquina;
   Begin
      Randomize;
      InitPantallaJuego;
      PintarLineaH2('', Color(VerdeClaro, Negro), 17, 40, 18);

      Repeat
         InitJugador(Jugador, 0);
         InitMaquina(Maquina);

         Repeat
            Turno(Jugador, Maquina.Tablero, Maquina.NumBarcos);
            If Maquina.NumBarcos > 0 Then
               TurnoMaquina(Maquina, Jugador);
         Until (Jugador.NumBarcos <= 0) Or (Maquina.NumBarcos <= 0);

         If Maquina.NumBarcos = 0 Then
          Begin
            Vencedor;
            AnyadirPuntuacion(Jugador.Nombre, Jugador.Disparos);
          End
         Else
            Perdedor;
      Until OtraMas(1) = 'N';
   End;

{**************************************************************************}

   Procedure DosJugadores;
   Var
      Jugador1, Jugador2 : TJugador;
   Begin
      Randomize;
      InitPantallaJuego;

      Repeat
         PintarTexto('1', Color(CyanClaro, Negro), 17, 40);
         InitJugador(Jugador1, 1);
         PintarTexto('2', Color(CyanClaro, Negro), 17, 40);
         InitJugador(Jugador2, 2);
         PintarTexto('1', Color(CyanClaro, Negro), 17, 40);

         Repeat
            Turno(Jugador1, Jugador2.Tablero, Jugador2.NumBarcos);
            If Jugador2.NumBarcos > 0 Then
             Begin
               CambioTurno(1);

               Turno(Jugador2, Jugador1.Tablero, Jugador1.NumBarcos);
               If Jugador1.NumBarcos > 0 Then
                  CambioTurno(2);
             End;
         Until (Jugador1.NumBarcos <= 0) Or (Jugador2.NumBarcos <= 0);

         If Jugador2.NumBarcos = 0 Then
          Begin
            Vencedor2(1);
            AnyadirPuntuacion(Jugador1.Nombre, Jugador1.Disparos);
          End
         Else
          Begin
            Vencedor2(2);
            AnyadirPuntuacion(Jugador2.Nombre, Jugador2.Disparos);
          End;
      Until OtraMas(2) = 'N';
   End;

{**************************************************************************}

End.

{**************************************************************************}
{                         Fin de la unidad Juego                           }
{**************************************************************************}