{******************************************************************************************}
{ "Hundir la Flota", recreacion del clasico juego de tablero de guerra maritima.           }
{ Copyright (C) 2002  Gorka Surez Garca                                                  }
{                                                                                          }
{ Puntos.pas is part of "Hundir la Flota".                                                 }
{                                                                                          }
{ "Hundir la Flota" is free software; you can redistribute it and/or                       }
{ modify it under the terms of the GNU General Public License                              }
{ as published by the Free Software Foundation; either version 2                           }
{ of the License, or (at your option) any later version.                                   }
{                                                                                          }
{ "Hundir la Flota" is distributed in the hope that it will be useful,                     }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of                           }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            }
{ GNU General Public License for more details.                                             }
{                                                                                          }
{ You should have received a copy of the GNU General Public License                        }
{ along with "Hundir la Flota"; if not, write to the Free Software                         }
{ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              }
{******************************************************************************************}

{**************************************************************************}
{ Puntos.pas                                                               }
{ Autor: Gorka Suarez Garcia                                               }
{ Fecha: 05-02-02                                                          }
{**************************************************************************}
Unit Puntos;
Interface {Parte publica}
   Uses Crt, Graflib;

   {Rutina para aadir una puntuacion}
   Procedure AnyadirPuntuacion (Nombre : String; Disparos : Byte);
   {Con esta rutina se listan las puntuaciones}
   Procedure VerPuntuaciones;

{**************************************************************************}

Implementation {Parte privada}
   Const
      NFICH   = 'Puntos.dat'; {Nombre del archivo para guardar las 
                               puntuaciones}
      MAXPUNT = 16; {Numero maximo de puntuaciones que se pueden guardar}

   Type
      {Tipos para manejar las puntuaciones}
      TPuntuacion = Record
         Nombre : String[16];
         Disparos : Byte;
      End;
      TPuntuaciones = array [1..MAXPUNT] of TPuntuacion;
      TFichero = File of TPuntuaciones;

{----- Variables globales -----}
   Var
      Puntuaciones : TPuntuaciones; {Array con las puntuaciones}
      FPunt        : TFichero; {Manejador del fichero}


{**************************************************************************}
{----------------------- Funciones y procedimientos -----------------------}
{**************************************************************************}

   Procedure AnyadirPuntuacion (Nombre : String; Disparos : Byte);
   Var
      i : Byte;
   Begin
      i := MAXPUNT;
      {Para aadir una puntuacion esta debera ser minimamente, igual a la
       puntuacion de la ultima posicion que se haya guardado}
      If Disparos <= Puntuaciones[i].Disparos Then
       Begin
         {Este while sirve para buscar la posicion en la que se insertara
          la nueva puntuacion. Recorre el array desde el final al principio,
          para ver en que posicion ha de insertar la nueva puntuacion.
          A medida que va recorriendo el array, va echando una posicion
          hacia atras los registros, para que haya un hueco donde meter la
          nueva puntuacion. Las puntuaciones estan ordenadas de forma
          Ascendente}
         While (Disparos <= Puntuaciones[i - 1].Disparos) And (i > 1) Do
          Begin
            Puntuaciones[i] := Puntuaciones[i - 1];
            i := i - 1;
          End;
         {Ahora que ha encontrado la posicion, mete la nueva puntuacion}
         Puntuaciones[i].Nombre := Nombre;
         Puntuaciones[i].Disparos := Disparos;

         {Y se acutaliza el fichero}
         Reset(FPunt);
         Write(FPunt, Puntuaciones);
         Close(FPunt);
       End;
   End;

{**************************************************************************}

   Procedure VerPuntuaciones;
   Var
      i : Byte;
   Begin
      {Borrado de pantalla}
      CambiarColor(Color(VerdeClaro, Negro));
      ClrScr;

      {Cabecera}
      GotoXY(24, 13); Write('********************************');
      GotoXY(24, 14); Write('*                              *');
      GotoXY(24, 15); Write('*   Las mejores puntuaciones   *');
      GotoXY(24, 16); Write('*                              *');
      GotoXY(24, 17); Write('********************************');

      {Listado de los datos}
      For i := 1 To MAXPUNT Do
       Begin
         GotoXY(29, 20 + i); Write(Puntuaciones[i].Nombre);
         GotoXY(45, 20 + i); Write(' - ', Puntuaciones[i].Disparos:3);
       End;

      {Pausa}
      GotoXY(1, UltF);
      ReadKey;
   End;

{**************************************************************************}

   {Este modulo es llamado al principio para inicializar el array de
    puntuaciones. Si existe Puntos.dat, se cargan las puntuaciones
    almacenadas de este, en el array, si no existe, se inicializa solo.}
   Procedure InitPuntuaciones;
   Var
      i : Byte;
   Begin
      Assign(FPunt, NFICH);
      {$I-}
      Reset(FPunt);
      {$I+}
      If IOResult = 0 Then {El fichero existe}
         Read(FPunt, Puntuaciones)
      Else {El fichero no existe}
       Begin
         Rewrite(FPunt);
         {Llenado automatico del array}
         For i := 1 To MAXPUNT Do
          Begin
            Puntuaciones[i].Nombre := 'Nadie todavia';
            Puntuaciones[i].Disparos := 100;
          End;
         Write(FPunt, Puntuaciones);
       End;
      Close(FPunt);
   End;

{**************************************************************************}

Begin {Constructor}
   InitPuntuaciones;
End.

{**************************************************************************}
{                         Fin de la unidad Puntos                          }
{**************************************************************************}